function [Rsqout,RsqConcout,finalMassBalanceErrorout]=Fix_Piv_Code_Slave_Function_for_Fitting(alfcal,Dfcal,deltfacin)
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%            A code for calibration of model parameters                   % 
%                           developed by                                  %
%                         Peyman Babakhani                                % 
%                    University of Liverpool, UK                          %
%               National Tsing Hua University, Taiwan                     %
%                    p.babakhani@liverpool.ac.uk                          %
%                        peyman.bi@yahoo.com                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %      
%   This code is based upon the Fixed Pivot approach developed by Kumar   %
%   and Ramkrishna, 1995. The code is developed for simulating early      % 
%   and late stages of aggregation and sedimentation of nanoparticles.    % 
%                                                                         %
%   This script serves as a "slave model" that solves the Fixed Pivot     % 
%   model, for aggregation and sedimentation model under the "master      %  
%   model" which is named "Parameter Estimator" for calibration of        % 
%   parameters.                                                           % 
%                                                                         % 
%   This code is generally similar to the code which also plots the       % 
%   results.                                                              %
%   All the units are in the SI system. The input and output sizes are    % 
%   converted to nm.                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!%
%   IMPORTANT NOTE: The user can use and modify this code for any         % 
%   non-commercial purposes with proper reference to the following paper: %
%   Peyman Babakhani, Jonathan Bridge, Tanapon Phenrat, Ruey-an Doong,    %   
%   and Karl R Whittle, 2018 "Aggregation and sedimentation of shattered  % 
%   graphene oxide nanoparticles (SGO) in dynamic environments: a         % 
%   solid-body rotational approach"                                       %
%   Environmental, Science: Nano                                          %
%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
tic;
%
 % Inputs:
%
%Assign the output file names:
FileName='GO_Cylinder_Fix_Piv_Brinkman_PowerLaw';
 
%Assign the number of plots, or the number of experimental cases :
PlotNo=1;
 
%Assign the case number of the input data which is going to be modelled. If PlotNo>1, 
%then caseNo will be ignored:
caseNo=4;
%_____________________________________________________
 
%Parameters:
%Assign the attachment efficiency:
%alfMat=[0 0.0008 1.0e-5 1e-5 0.9952e-3 0.9952*1e-3 0.9952*1e-3 0.9952*1e-3 0.9952*1e-3 0.9952*1e-3 0.9952*1e-3 14];
alfMat=alfcal;
%Assign the fractal Dimension:
%Elements should be as many as the number of experimental cases.
%DfMat=[2.2000 2.999 2.5 2.4 1.7 1.7 1.7 1.7 1.7 1.7 1.7 2.58];
DfMat=Dfcal;
%Assign the factor for refining the initial delta_t:
%Elements should be as many as the number of experimental cases.
%deltfacMat=[2 2000 2000 100 10 10 10 10 10 10 10 400];
deltfacMat=deltfacin;
 
%Assign time step magnification:
%Elements should be as many as the number of experimental cases.
%This factor is used to increase the time step after passing the early stage 
%of the modelling, where the model run is computationally problematic. Usually
%a value of 1000 is appropriate unless the running is slow due to slow
%aggregation, then this value can be increased, e.g., to 1000000
deltmagMat=[1000000 1000 1000 1000 1000 1000 1000 1000 1000 1000];
 
%Assign the portion of the simulation time period after which the time step 
%can be increased:
RelaxationPortion=1/40;
 
%Delta t factor after relaxation:
deltfacMatRelax=2;
 
%Assign the initial Concentration (mg/L)
c0=50;  %mg/L
c0=c0/1000;  %kg/m^3
 
%Assign the minimum initial hydrodynamic (Z-average) diameter observed among
%all the experimental cases (nm):
D0real=90;       %nm
D0real=D0real*10^-9;
 
%Assign the density of solids particles and the fluid (kg/m^3)
RhoS=1800;  % kg/m^3   for GO
%RhoS=3160;   % kg/m^3   for HAp
Rho=1000;    % kg/m^3   for water 
 
%Assign the experiment duration (min):
% By default, the run time is automatically selected from the experimental
% data. However, if you wish to run the model for shorter times then assign
% a value to in the following line other than 1:
T_user=1;  % min
 
%PSD snapshots time for early stage:
TT1=30;
TT1=TT1*60;  % s
 
%PSD snapshots time for late stage:
TT2=100;
TT2=TT2*60;  % s
 
%Assign the number of delta_x, or the number of delta_x that should be considered:
%Here, in order to save time of modelling, we assign this as one. Otherwise,
%since it does not affect the numerical method, selection of 10-20 intervals
%is adequate for appropriate resolution of the curve. 
ndelx=1;
%ndelx=ndelx+1;
 
%Assign the vial depth or the total depth of the water column from the surface of 
% water to the bottom of the column:
%Elements should be as many as the number of experimental cases.
dsMat=[3 3 3 3 3 3 3 3 3 3 3 3];
 
%Cross section area of the vial:
Across=0.01*0.01; %m^2  %for common cuvettes
 
%Measurement depth (position) of measurement(Should be smaller than dsMat)
%This is the depth where aggregation is monitored
%Elements should be as many as the number of experimental cases.
%MeasDep=[2.5 2.5 2.5 2.5 2.5 2.5 2.5 2.5 2.325 2.325 2.325 2.325];   %cm
MeasDep=[2.5 2.5 2.5 2.5 2.325 2.325 2.325];   %cm
 
%Shear Rate
%Can be used to simulate the condition of the vortex before the start of 
%the measurement:
G=0;  %  26.7; %(1/s) 
 
%Tempreture (kelvin):
Temp=298;  %'K
 
%Viscosity:
Visc=0.00089;  %Pa.s or Kg/m/s 
 
%If ndelx>1, assign the number of plots that should be plotted for 
%concentration change along the height in the course of the experiment
plotnumber=6;
 
%Factor for further resolution of the particle size dimension discretization refinement:
%This helps obtaining higher resolution in the PSD.
%Elements should be as many as the number of experimental cases.
bin_resolut_facMat=[1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1];
 
%The total number of bins that is considered in the simulation:
%For DLS data of aggregation and sedimentation over the course of 5 hours,
%it can be considered as 100. Later this will be automatically decreased
%based on the initial PSD bins with non-zero concentration.
x_dist_data_No=100;
 
%Particle shape factor of nanoparticles for calculation of the a sedimentation velocity
%using the particles size distribution model of Kehlifa and Hill:
teta=1;    %assuming spherical particles
 
%??????????????????????????????????????????????????
%Select the collision frequency calculation method:
%??????????????????????????????????????????????????
%Collision_Calculation_mehtod=1; ----> permeability method based on Brinkman model
%Collision_Calculation_mehtod=2; ----> permeability method based on Davies Correlation
%Collision_Calculation_mehtod=3; ----> simple fractal model without permeability 
Collision_Calculation_mehtod=1;
 
%???????????????????????????????????????
%Select the velocity calculation method:
%???????????????????????????????????????
%Vel_Calculation_mehtod=1; ----> power law method
%Vel_Calculation_mehtod=2; ----> permeability method based on Davies Correlation
%Vel_Calculation_mehtod=3; ----> permeability method based on Brinkman model
%Vel_Calculation_mehtod=4; ----> particle size distribution-based model
Vel_Calculation_mehtod=1;
 
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%Reading experimental data:
Input1a=load('Input1.mat');
Input1b= struct2cell(Input1a);
Input1=cell2mat(Input1b);
 
Input2a=load('Input2.mat');
Input2b= struct2cell(Input2a);
Input2=cell2mat(Input2b);
 
x_initial_size_dista=load('x_initial_size_dist.mat');
x_initial_size_distb= struct2cell(x_initial_size_dista);
x_initial_size_dist=cell2mat(x_initial_size_distb);
 
obs_size_dista=load('obs_size_dist.mat');
obs_size_distb= struct2cell(obs_size_dista);
obs_size_dist=cell2mat(obs_size_distb);
 
 
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%% End of User Inputs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%Readig the particle size distributions at the beginning, middle and the end
%of the experiments and finding the minimum bin size 
OutputFileName=FileName;
for pn=1:PlotNo
    
    if PlotNo>1
        caseNo=pn;
    end
%Reading the observed distribution and initial distribution:
   InputDistInitial(pn,:)=obs_size_dist(1:x_dist_data_No,3*caseNo-2);
   InputDistMid(pn,:)=obs_size_dist(1:x_dist_data_No,3*caseNo-1);
   InputDistEnd(pn,:)=obs_size_dist(1:x_dist_data_No,3*caseNo);
 
%Finding the minimum bin size of the initial particle size distribution
%where the concentration is nonzero. This will be considered
%as the first bin size in the model:
        first_non_zero1=find(InputDistInitial(pn,:));
        first_non_zero2=find(InputDistMid(pn,:));
        first_non_zero3=find(InputDistEnd(pn,:));
        Initmin=first_non_zero1(1);
        midmin=first_non_zero2(1);
        Endmin=first_non_zero3(1);
        tempindx=Initmin;
        D01Mat(pn)=x_initial_size_dist(tempindx);  %nm
        D01Mat(pn)=D01Mat(pn)*10^-9;                %m
bin_resolut_fac=bin_resolut_facMat(pn);
kk=x_dist_data_No-tempindx;
kk=kk*bin_resolut_fac;
kkk(pn)=kk; %This is the number of required bins that is used in each case of modelling
intiateBinNoMat(pn)=x_dist_data_No*bin_resolut_fac-kkk(pn)+1;
 
end
 
size_ditrib1=zeros(PlotNo,max(kkk));
No_size_ditrib1=zeros(PlotNo,max(kkk));
Mass_size_dist1=zeros(PlotNo,max(kkk));
 
size_ditrib2=zeros(PlotNo,max(kkk));
No_size_ditrib2=zeros(PlotNo,max(kkk));
Mass_size_dist2=zeros(PlotNo,max(kkk));
 
size_ditrib3=zeros(PlotNo,max(kkk));
No_size_ditrib3=zeros(PlotNo,max(kkk));
Mass_size_dist3=zeros(PlotNo,max(kkk));
 
 
%Reading the data number in each case:
for pn=1:PlotNo
    
        if PlotNo>1
        caseNo=pn;
        end
    
Input11=0;
Input11=Input1(:,3*caseNo-2);
%Removign NaN from the matrix 
Input11(isnan(Input11))=[];
siztemp=size(Input11);
dataNo(pn)=siztemp(1);
end
 
ConcOutputMat=zeros(PlotNo,max(dataNo));
ConcMat=zeros(PlotNo,max(dataNo),ndelx);
ModelMat=zeros(PlotNo,max(dataNo));
 
set(0,'defaultAxesFontName', '<Arial>')
set(0,'defaultTextFontName', '<Arial>')
 
markers=['o', 's', 'd', '^', 'v', '>','<','h', 'o', 's', 'd', '^', 'v', '>','<','h'];
%% 
%##########################################################################
%?????????????????????????????????????????????????????????????????????????%
%The start of the loop for the simulating every experimental data set until
%plotting the hydrodynamic size evolution 
%?????????????????????????????????????????????????????????????????????????%
for pn=1:PlotNo
   tic; 
Inputx=0;
Inputy=0;
if PlotNo>1
    caseNo=pn;
end
    
%Input variables from the input data:
    Inputx=Input1(1:dataNo(pn),3*caseNo-2);
    Inputy=Input1(1:dataNo(pn),3*caseNo-1);
    InputErr=Input1(:,3*caseNo);
    InputEr=InputErr;
    L=Inputy(:)<20;
    Inputx(L)=[];
    Inputy(L)=[];
    InputEr(L)=[];
    sizI=size(InputEr);
%number of NaN in the matrix 
    nan=isnan(InputEr);
    nnan=nnz(nan);
    net=sizI(1)-nnan;
    panj=[2*pn:5:net];
    newInputEr=zeros(sizI(1));
    panjmat=InputEr(panj);
    newInputEr(sub2ind(size(InputEr), panj'))=panjmat;
    Dgeom=0;
  
%Finding the duration of run:
  if T_user==1
   T=max(Inputx);% min
  else
   T=T_user;% min
  end
   T=T*60;  % s
   
%Selecting the Minimum possible Delta_t as around 1/1000 times of the initial Deleta_t:
    mindelt=T/100/max(deltfacMat)*60/1000;
 
    alf=alfMat(1);
    Df=DfMat(1);
    deltfac=deltfacMat(1);
    deltmag=deltmagMat(1);
 
    ds=dsMat(1);
    ds=ds/100;  %m
%Calculation of delta_x
    delx=ds/(ndelx);
 
mdp=MeasDep(caseNo); %cm for the beam position in the cuvette placed in the Malvern Zetasizer DLS
mdp=mdp/100;  %m
mdpNdx=round(mdp/delx); %Cell index, which is the place of the measurement.
 
% If the number of delta_x is one then:
if ndelx==1
mdpNdx=1;
delx=mdp;
end
 
%Attributing delta t:
%deltfac=deltfacMat(pn);
deltinitial=3.3678;  %min
deltinitial=deltinitial*60;  % s
delt1=deltinitial/deltfac; 
 
ndelt0=(round(T/deltinitial)+1);
if ndelt0>dataNo(pn)
    ndelt0=dataNo(pn); 
end
    ndelt1=ndelt0*deltfac;
    
%if ndelt1>dataNo(1)*deltfac
%    ndelt1=dataNo(1)*deltfac; 
%end
 
bin_resolut_fac=bin_resolut_facMat(pn);
 
c0bin=0;
a=0;
n=0;
vsMat=zeros(ndelx,kkk(pn));
vsMat1=zeros(ndelx,kkk(pn));
vssM=0;
aMat=zeros(ndelx,kkk(pn));
aMat1=zeros(ndelx,kkk(pn));
c=zeros(ndelx,kkk(pn));
ms=zeros(ndelx,kkk(pn));
nMat=zeros(ndelx,kkk(pn));
u=zeros(kkk(pn));
VMatinit=zeros(1,kkk(pn));
radii_dist_init=zeros(kkk(pn));
c_dist_init=zeros(1,kkk(pn));
n_dist_init=zeros(1,kkk(pn));
radii_dist_init_res=0;
radii_dist_init_res1=0;
InputDistInitial_res=0;
InputDistInitial_res1=0;
Beta1=0;
dnx=zeros(ndelx,kkk(pn));
dnt=zeros(ndelx,kkk(pn));
 
 
  %Assign the Initial Delta_t:
 % tempndelt=round(T/60/3.3678)-2;
for i=1:ndelt0-1
deltinitialM(i)=(Inputx(i+1)-Inputx(i));  %min     %This is the interval used in the DLS data 
end

%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
   D0=D01Mat(pn);
 
xdata=transpose(Inputx(:,1));
if max(Inputx)<299
    display('Error of lower time matrix');
    ErrorAt=pn
    shortage=299-max(Inputx)
end
 
%Determination of the start of the hydrodynamic diameter from the 
%initial condition of the experimental data:
DstTemp=Inputy(1:3,1);
DstTemp2=DstTemp(DstTemp(:)>40);
Dstart=min(DstTemp2);      %nm
Dstart=Dstart*10^-9;
 
Dstartdist=sum(x_initial_size_dist(1:x_dist_data_No).*...
    transpose(InputDistInitial(pn,1:x_dist_data_No)))./sum...
    (transpose(InputDistInitial(pn,1:x_dist_data_No)));
Dstartdist=Dstartdist*10^-9;
 
%Number of bins
kk=kkk(pn);
intiateBinNo=intiateBinNoMat(pn);
VMatinit(1:kk)=0;
a(1:kk)=0;
n(1:kk)=0;
c0bin(1:kk)=0;
u(1:kk)=0;
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
%Initial total Mass:
massInitial=Across*ds*c0;           %kg 
 
% The radius of the minimum of the size distribution bins with non-zero concentration
a01=D0/2;  
 
%Assuming that for primary particles gyration radius can be obtained from DLS size based on
%Henry, Norrfors et al. 2015:
%The radius of the minimum hydrodynamic size observed among all experiments
a0real=D0real/2*gamma(2-Df^-1); 
 
%Selecting the final primary size based on the minimum of the two above
%sizes:
a0=min(a0real,a01);
 
%Initial solid Volume of priamry particels (assuming spherical):
V0=4/3*3.1415*(a0)^3;
 
%Initial radius distribution:
radii_dist_init=x_initial_size_dist/2*1e-9;
 
%Distributing the sizes regarding the bin resolution factor:
for k=1:x_dist_data_No
   radii_dist_init_res1(k)=radii_dist_init(k);
   InputDistInitial_res1(k)=InputDistInitial(pn,k)/bin_resolut_fac;
 
end
%
 
indkj=0;
for k=1:x_dist_data_No
    for j=1:bin_resolut_fac
  indkj=indkj+1;
       if k==1
    radii_dist_init_res(indkj)=radii_dist_init_res1(1);
    InputDistInitial_res(indkj)=InputDistInitial_res1(1);
       else
        if indkj<=x_dist_data_No*bin_resolut_fac
    radii_dist_init_res(indkj)=radii_dist_init_res1(k-1)+j*(radii_dist_init_res1(k)-...
        radii_dist_init_res1(k-1))/bin_resolut_fac;
    InputDistInitial_res(indkj)=InputDistInitial_res1(k-1)+j*(InputDistInitial_res1(k)-...
        InputDistInitial_res1(k-1))/bin_resolut_fac;
         end
       end
    end
end
 
%Initial mass distribution assuming the particle density times the bin size times volume percentage divided by 
%the sum of percentages (100%):
c_dist_init=c0*InputDistInitial_res(intiateBinNo:bin_resolut_fac*x_dist_data_No)/...
    sum(InputDistInitial_res(intiateBinNo:bin_resolut_fac*x_dist_data_No));
 
%Initial Population of particles:
nt0=c0/(RhoS*V0);
 
%Size, volume, initial number concentration, and initial mass concentration
%of each bin that will be used in the simulation from here on:
for k=1:kk
a(k)=radii_dist_init_res(k+intiateBinNo-1);
vsMat(1:ndelx,k)=3.1415/6*(2*a0)^(3-Df)*(a(k)*2).^Df;
n((k))=c_dist_init(k)/RhoS./vsMat(1,k);
c0bin((k))=c_dist_init(k);
end
 
 
for xx=1:ndelx;
for k=1:kk
vsMat1(xx,k)=vsMat(xx,(k));   %Volume matrix
aMat(xx,k)=a(k);              %radius matrix
ms(xx,k)=RhoS*vsMat(xx,k);    %mass matrix
nMat(xx,k)=n(k);              %number concentration matrix
end
end
 
rstartdist2number=sum(a.*n)/sum(n);
rstartdist2mass=sum(a.*c0bin)/sum(c0bin);
 
%Calculation of geometric average in the initial time:
logtemp(1:kk)=log(2*aMat(1,1:kk));
Dgeom(1,1)=exp(sum(c0bin(1:kk).*(2*aMat(1,1:kk)).^3.*logtemp(1:kk))/(sum(c0bin(1:kk).*(2*aMat(1,1:kk)).^3)));
 
%Determining the ratio of the initial hydrodynamic size to the geometric 
%mean of the model output 
Init_Computed_Dstart2=rstartdist2mass*2; 
Init_Computed_DstartDeom=Dgeom(1,1);
 
LaterSizeTransRatio=Dstart/Init_Computed_Dstart2;
LaterSizeTransRatioGeom(pn)=Dstart/Init_Computed_DstartDeom;
 
LaterSizeTransRatioMat(pn)=LaterSizeTransRatio;
 
LaterSizeTransRatio=LaterSizeTransRatioGeom(pn);
 
%% Calculation of sedimentation velocity with various approaches
%======================================================================================================
%For the purpose of plotting, we first calculate these velocities for three 
%Df values (1.6, 2, 2.3) and then for the Df the current modelling
%problem
 
Df=1.6;
u0=2*(RhoS-Rho)*9.81*a0^2/9/Visc;
%Power Law:
DSed=Df-1;
for k=1:kk
u1(k)=u0*(aMat(1,k)./a0).^(DSed);
 
porosity(k)=1-(a(k)/a0)^(Df-3);
 
%Permeability from Davies Correlation:
permeabDa(k)=1/(16/(a(k))^2*(1-porosity(k))^1.5*(1+56*(1-porosity(k))^3));
 
kesiDa(k)=a(k)/(permeabDa(k))^0.5;
 
omegDa(k)=(2*(kesiDa(k))^2*(1-tanh(kesiDa(k))/kesiDa(k)))/(2*(kesiDa(k))^2+3*(1-tanh(kesiDa(k))/kesiDa(k)));
 
 
 
%Velocity from Jhonson, Li and Logan (1996):
kesi=kesiDa;
uLogan1(k)=2*(RhoS-Rho)*9.81*a(k)^2/9/Visc*(kesi(k)/(kesi(k)-tanh(kesi(k)))+3/(2*(kesi(k))^2));
 
 
%Permeability from Brinkman:
permeabBr(k)=(2*a0)^2/72*(3+3/(1-porosity(k))-(8/(1-porosity(k))-3)^(1/3));
 
kesiBr(k)=a(k)/(permeabBr(k))^0.5;
 
omegBr(k)=(2*(kesiBr(k))^2*(1-tanh(kesiBr(k))/kesiBr(k)))/(2*(kesiBr(k))^2+3*(1-tanh(kesiBr(k))/kesiBr(k)));
 
phi=porosity(k);
% Velocity from Thill et al. (Wiesner), (2001) and Vikesland et al., 2016 
%(This is the same equation with that of Li and Logan (1996), only the permeability is different):):
uWiesner1(k)=(RhoS-Rho)*9.81*phi*(2*a(k))^2/18/Visc/omegBr(k);
 
%}
end
 
% Velocity based on size distribution from Khelifa and Hill (2006) cited by Vahedi and Gorczyca (2014):
 
    Re1=1.0293e-06;
    Re2=2.0293e-06;
    d50=0;
    m3=0;
    mF=0;
    phi=0;
 %
for k=1:kk
uKhelif1=0;
Re1=1.0293e-06;
 
    m3(k)=sum((2*a(1:k)).^3)/k;
    mF(k)=sum((2*a(1:k)).^Df)/k;
    phi(k)=m3(k)/(mF(k))^(3/Df);
    
   d50(k)=2*a(round(k/2));
 
   while abs(Re2-Re1)/Re2>0.01
    Re1=Re2;
    uKhelif1=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
    Re2=uKhelif1*2*a(k)*Rho/Visc;
   end
   
uKhel1(k)=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
 
end
%==================================================================================================
 
%======================================================================================================
 
Df=2.0;
u0=2*(RhoS-Rho)*9.81*a0^2/9/Visc;
DSed=Df-1;
for k=1:kk
  %Power Law:  
u2(k)=u0*(aMat(1,k)./a0).^(DSed);
 
porosity(k)=1-(a(k)/a0)^(Df-3);
 
%Permeability from Davies Correlation:
permeabDa(k)=1/(16/(a(k))^2*(1-porosity(k))^1.5*(1+56*(1-porosity(k))^3));
 
kesiDa(k)=a(k)/(permeabDa(k))^0.5;
 
omegDa(k)=(2*(kesiDa(k))^2*(1-tanh(kesiDa(k))/kesiDa(k)))/(2*(kesiDa(k))^2+3*(1-tanh(kesiDa(k))/kesiDa(k)));
 
%Velocity from Jhonson, Li and Logan (1996):
kesi=kesiDa;
uLogan2(k)=2*(RhoS-Rho)*9.81*a(k)^2/9/Visc*(kesi(k)/(kesi(k)-tanh(kesi(k)))+3/(2*(kesi(k))^2));
 
%Permeability from Brinkman:
permeabBr(k)=(2*a0)^2/72*(3+3/(1-porosity(k))-(8/(1-porosity(k))-3)^(1/3));
 
kesiBr(k)=a(k)/(permeabBr(k))^0.5;
 
omegBr(k)=(2*(kesiBr(k))^2*(1-tanh(kesiBr(k))/kesiBr(k)))/(2*(kesiBr(k))^2+3*(1-tanh(kesiBr(k))/kesiBr(k)));
 
phi=porosity(k);
% Velocity from Thill et al. (Wiesner), (2001) and Vikesland et al., 2016 
%(This is the same equation with that of Li and Logan (1996), only the permeability is different):
uWiesner2(k)=(RhoS-Rho)*9.81*phi*(2*a(k))^2/18/Visc/omegBr(k);
 
end
 
%
% Velocity based on size distribution from Khelifa and Hill (2006) cited by Vahedi and Gorczyca (2014):
 
    Re1=1.0293e-06;
    Re2=2.0293e-06;
    d50=0;
    m3=0;
    mF=0;
    phi=0;
 %
for k=1:kk
uKhelif2=0;
Re1=1.0293e-06;
 
    m3(k)=sum((2*a(1:k)).^3)/k;
    mF(k)=sum((2*a(1:k)).^Df)/k;
    phi(k)=m3(k)/(mF(k))^(3/Df);
 
   d50(k)=2*a(round(k/2));
 
   while abs(Re2-Re1)/Re2>0.01
    Re1=Re2;
    uKhelif2=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
    Re2=uKhelif2*2*a(k)*Rho/Visc;
   end
uKhel2(k)=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
 
end
%==================================================================================================
 
%======================================================================================================
Df=2.3;
u0=2*(RhoS-Rho)*9.81*a0^2/9/Visc;
%Power Law:
DSed=Df-1;
for k=1:kk
u3(k)=u0*(aMat(1,k)./a0).^(DSed);
 
porosity(k)=1-(a(k)/a0)^(Df-3);
 
%Permeability from Davies Correlation:
permeabDa(k)=1/(16/(a(k))^2*(1-porosity(k))^1.5*(1+56*(1-porosity(k))^3));
 
kesiDa(k)=a(k)/(permeabDa(k))^0.5;
 
omegDa(k)=(2*(kesiDa(k))^2*(1-tanh(kesiDa(k))/kesiDa(k)))/(2*(kesiDa(k))^2+3*(1-tanh(kesiDa(k))/kesiDa(k)));
 
%Velocity from Jhonson, Li and Logan (1996):
kesi=kesiDa;
uLogan3(k)=2*(RhoS-Rho)*9.81*a(k)^2/9/Visc*(kesi(k)/(kesi(k)-tanh(kesi(k)))+3/(2*(kesi(k))^2));
 
%Permeability from Brinkman:
permeabBr(k)=(2*a0)^2/72*(3+3/(1-porosity(k))-(8/(1-porosity(k))-3)^(1/3));
 
kesiBr(k)=a(k)/(permeabBr(k))^0.5;
 
omegBr(k)=(2*(kesiBr(k))^2*(1-tanh(kesiBr(k))/kesiBr(k)))/(2*(kesiBr(k))^2+3*(1-tanh(kesiBr(k))/kesiBr(k)));
 
phi=porosity(k);
 
% Velocity from Thill et al. (Wiesner), (2001) and Vikesland et al., 2016 
%(This is the same equation with that of Li and Logan (1996),only the permeability is different):):
uWiesner3(k)=(RhoS-Rho)*9.81*phi*(2*a(k))^2/18/Visc/omegBr(k);
 
end
 
% Velocity based on size distribution from Khelifa and Hill (2006) cited by Vahedi and Gorczyca (2014):
 
    Re1=1.0293e-06;
    Re2=2.0293e-06;
    d50=0;
    m3=0;
    mF=0;
    phi=0;
 %
for k=1:kk
uKhelif3=0;
Re1=1.0293e-06;
 
    m3(k)=sum((2*a(1:k)).^3)/k;
    mF(k)=sum((2*a(1:k)).^Df)/k;
    phi(k)=m3(k)/(mF(k))^(3/Df);
    
   d50(k)=2*a(round(k/2));
 
   while abs(Re2-Re1)/Re2>0.01
    Re1=Re2;
    uKhelif3=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
    Re2=uKhelif3*2*a(k)*Rho/Visc;
   end
uKhel3(k)=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
 
end
%==================================================================================================
 
%The current simulation case Df:
%Df======================================================================================================
u0=2*(RhoS-Rho)*9.81*a0^2/9/Visc;
Df=DfMat(1);
 
%Power Law:
DSed=Df-1;
for k=1:kk
u(k)=u0*(aMat(1,k)./a0).^(DSed);
 
porosity(k)=1-(a(k)/a0)^(Df-3);
 
%Permeability using Happel cell model:
pack=0.1;
Rhob(k)=pack*(a(k)/a0)^(Df-3);
%permeab(k)=2*a0^2/9/Rhob(k)*(3-4.5*(Rhob(k))^(1/3)+4.5*(Rhob(k))^(5/3)-3*(Rhob(k))^2)/(3+2*(Rhob(k))^(5/3));
 
%Permeability from Davies Correlation:
permeabDa(k)=1/(16/(a(k))^2*(1-porosity(k))^1.5*(1+56*(1-porosity(k))^3));
 
kesiDa(k)=a(k)/(permeabDa(k))^0.5;
 
omegDa(k)=(2*(kesiDa(k))^2*(1-tanh(kesiDa(k))/kesiDa(k)))/(2*(kesiDa(k))^2+3*(1-tanh(kesiDa(k))/kesiDa(k)));
 
%Velocity from Jhonson, Li and Logan (1996):
kesi=kesiDa;
uLogan(k)=2*(RhoS-Rho)*9.81*a(k)^2/9/Visc*(kesi(k)/(kesi(k)-tanh(kesi(k)))+3/(2*(kesi(k))^2));
 
%Permeability from Brinkman:
permeabBr(k)=(2*a0)^2/72*(3+3/(1-porosity(k))-(8/(1-porosity(k))-3)^(1/3));
 
kesiBr(k)=a(k)/(permeabBr(k))^0.5;
 
omegBr(k)=(2*(kesiBr(k))^2*(1-tanh(kesiBr(k))/kesiBr(k)))/(2*(kesiBr(k))^2+3*(1-tanh(kesiBr(k))/kesiBr(k)));
 
phi=porosity(k);
 
% Velocity from Thill et al. (Wiesner), (2001) and Vikesland et al., 2016 
%(This is the same equation with that of Li and Logan (1996),only the permeability is different):
uWiesner(k)=(RhoS-Rho)*9.81*phi*(2*a(k))^2/18/Visc/omegBr(k);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Selecting the permeability model (Brinkman or Davies) for collision rate
%calculations:
%frequencies
if Collision_Calculation_mehtod==1
kesi(k)=kesiBr(k);
omeg(k)=omegBr(k);
elseif Collision_Calculation_mehtod==2
kesi(k)=kesiDa(k);
omeg(k)=omegDa(k);
elseif Collision_Calculation_mehtod==3
omeg(k)=1;
end
 
%Curvilinear Correction coefficients for collision efficiencies:
curvJ(k)=2*(kesi(k))^2+3-3*tanh(kesi(k))/kesi(k);
curvc(k)=-1/curvJ(k)*(kesi(k)^5+6*kesi(k)^3-tanh(kesi(k))/kesi(k)*(3*kesi(k)^5+6*kesi(k)^3));
curvd(k)=3/curvJ(k)*kesi(k)^3*(1-tanh(kesi(k))/kesi(k));
 
curvada(k)=1-curvd(k)/kesi(k)-curvc(k)/(kesi(k))^3;
 
if Collision_Calculation_mehtod==3
curvada(k)=1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
 
% Velocity based on size distribution from Khelifa and Hill (2006) cited by Vahedi and Gorczyca (2014):
 
    Re1=1.0293e-06;
    Re2=2.0293e-06;
    d50=0;
    m3=0;
    mF=0;
    phi=0;
 %
for k=1:kk
uKhelif1=0;
Re1=1.0293e-06;
 
    m3(k)=sum((2*a(1:k)).^3)/k;%   (NInitMat1(1,k));
    mF(k)=sum((2*a(1:k)).^Df)/k;%  (NInitMat1(1,k));
    phi(k)=m3(k)/(mF(k))^(3/Df);
    
   %d50(k)=(sum(vsMat(1,1:k)/k*6/(2*3.1415*(2*a0)^(3-Df))))^(1/Df);
   d50(k)=2*a(round(k/2));
 
   while abs(Re2-Re1)/Re2>0.01
    Re1=Re2;
    uKhelif1=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
    Re2=uKhelif1*2*a(k)*Rho/Visc;
   end
uKhel(k)=1/18*teta*9.81*(RhoS-Rho)/Visc*(d50(k))^(3-Df)*(2*a(k))^(Df-1)/(1+0.15*Re1^0.687)*phi(k);
 
end
%==================================================================================================
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Selecting the sedimentation velocity for collision frequencies:
if Vel_Calculation_mehtod==1
   % u is already specified as that calculated by the power law method
elseif Vel_Calculation_mehtod==2
    u=uLogan;
elseif Vel_Calculation_mehtod==3
    u=uWiesner;
elseif Vel_Calculation_mehtod==4
    u=uKhel;
end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
 
nMat1=nMat;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for xx=1:ndelx
c(xx,1:kk)=c0bin(1:kk);
c1(xx,1:kk)=c0bin(1:kk);
Conc1(1,xx)=1;
end
 
dnx(1:ndelx,1:kk)=0;
%delc(1:ndelx,1:kk)=0;
tt=1;
t=0;
nobs=1;
delt=delt1;
tMat(1)=0;
nsink(1:ndelx,1:kk)=0;
csink(1:ndelx,1:kk)=0;
xad(1:ndelx,1:kk)=0;
dnxadd(1:ndelx,1:kk)=0;
msinksum(1,1:ndelx)=0;
ppindex=0;
ppMat=0;
Cdnxadd(1:ndelx,1:kk)=0;
pp=0;
%uMat(1,1:ndelx,1:kk)=u(tt,1:ndelx,1:kk);
tstarttempMat(1)=0;
%tstartMat(1)=0;
st=1;
T1=T;
tamam0=0;
tamam=0;
tamam1=0;
 
vsxMat(1:ndelx,1:kk)=V0;
vsxMat1(1:ndelx,1:kk)=V0;
%nsxMat(1:ndelx,1:kk)=0;
%nMat1(1:ndelx,1:kk)=0;
vs(1:kk)=V0;
vx(1:kk)=V0;
ns(1:kk)=V0;
 
%{
for xx=1:ndelx
size_ave(1,xx)=sum(aMat(xx,:).*nMat(xx,:))/sum(nMat(xx,:));
end
Hydr_Diam(1,1:ndelx)=0;
%uMat(1,1:ndelx,1:kk)=u(1:ndelx,1:kk);
%}
 
% Extracting the data for size distribution at initial time
size_ditrib1(caseNo,1:kk/bin_resolut_fac)=aMat(mdpNdx,1:bin_resolut_fac:kk)*10^9*2;   %/gamma(2-(real(Df))^-1);   %nm
No_size_ditrib1(caseNo,1:kk/bin_resolut_fac)=nMat(mdpNdx,1:bin_resolut_fac:kk)/sum(nMat(mdpNdx,1:bin_resolut_fac:kk));
Mass_size_dist1(caseNo,1:kk/bin_resolut_fac)=c(mdpNdx,1:bin_resolut_fac:kk)/sum(c(mdpNdx,1:bin_resolut_fac:kk));
 
%Calculating the averaged size:
for xx=1:ndelx;
logtemp(1:kk)=log(2*aMat(1,1:kk));
Dgeom(1,xx)=exp(sum(c(1,1:kk).*(2*aMat(1,1:kk)).^3.*logtemp(1:kk))/(sum(c(1,1:kk).*(2*aMat(1,1:kk)).^3)))*LaterSizeTransRatio;
 
size_ave(1,xx)=Dgeom(1,xx)/2;
end
 
%((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((((
%% The start of the delta_t loop:
while t<=T1 
tt=tt+1;
t=t+delt;
 
 dnvx=0;
 ss1Mat=0;
ss2Mat=0;
sxs1Mat=0;
ss2Max=0; 
 
for xx=1:ndelx
    
    vs2(1:kk)=vsMat(xx,1:kk);
    vs(1:kk)=vsMat(xx,1:kk);
    vx(1:kk)=vsMat(xx,1:kk);
    nMat1(xx,1:kk)=nMat(xx,1:kk);
 
 
for k=1:kk
 
    s1=0;
    ss1=0;
    s2=0;
    ss2=0;
    sxs1=0;
    sx1=0;
 
    
 if k==1
Beta1(1)=0; 
   s1=0;
   ss1=0;
 elseif k<kk
 
for i=1:kk
        for j=1:i
            
    if ((vx(i)+vx(j))>=vsMat(xx,k-1) && (vx(i)+vx(j))<=vsMat(xx,k+1))
%Collision Frequency calculation for the first term in the aggregation model:
    Beta1(i)=2*1.381*10^(-23)*Temp/(3*Visc)*((vs(i))^(1/Df)+(vs(j))^(1/Df))...
        *((vs(i))^(-1/Df)/omeg(i)+(vs(j))^(-1/Df)/omeg(j))...
        +(G/3.1415*V0^(1-3/Df)*(vs(i)^(1/Df)*curvada(i)^0.5+vs(j)^(1/Df)*curvada(j)^0.5)^3) ...
         +3/2*(3.1415/6)^(1/3)*V0^(2/3-2/Df)*((vs(i))^(1/Df)*curvada(i)^0.5+...
         (vs(j))^(1/Df)*curvada(j)^0.5)^2*abs(u(i)-u(j));
 
    if ((vx(i)+vx(j))>=vsMat(xx,k) && (vx(i)+vx(j))<=vsMat(xx,k+1))
    ada(k)=(vsMat(xx,k+1)-(vx(j)+vx(i)))/(vsMat(xx,k+1)-vsMat(xx,k));
 
    elseif((vx(i)+vx(j))>=vsMat(xx,k-1) && (vx(i)+vx(j))<=vsMat(xx,k))
    ada(k)=((vx(j)+vx(i))-vsMat(xx,k-1))/(vsMat(xx,k)-vsMat(xx,k-1));
    end
    if i == j
    kroneckerDelta = 1;
    else
    kroneckerDelta = 0;
    end
%Summations of the Fixed Pivot population balance equation (Kumar and Ramkrishna, 1995:
        s1=(1-0.5*kroneckerDelta)*ada(k)*Beta1(i)*nMat1(xx,j)*nMat1(xx,i);
        ss1=ss1+s1;
 
   end
   end
 
end
 elseif k==kk
 
     for i=1:kk
        for j=1:i
         if ((vx(i)+vx(j))>=vsMat(xx,k-1) && (vx(i)+vx(j))<=vsMat(xx,k))
 
             %It is assumed that the front of the concentration does
             %not reach the last bin during aggregation and sedimentation
             %processes. With using 100 bins this assumption was correct in all
             %cases of simulations up to 5 hours of experimental courses
 
        ss1=ss1+s1;
        
         end
        end
     end
 end
 
for j=1:kk
    %Collision Frequency calculation for the second term in the aggregation model:
    Beta2(j)=2*1.381*10^(-23)*Temp/(3*Visc)*((vs2(k))^(1/Df)+(vs2(j))^(1/Df))...
        *((vs2(k))^(-1/Df)/omeg(k)+(vs2(j))^(-1/Df)/omeg(j))...
        +(G/3.1415*V0^(1-3/Df)*(vs2(k)^(1/Df)*curvada(k)^0.5+vs2(j)^(1/Df)*curvada(j)^0.5)^3) ...
         +3/2*(3.1415/6)^(1/3)*V0^(2/3-2/Df)*((vs2(k))^(1/Df)*curvada(k)^0.5+...
         (vs2(j))^(1/Df)*curvada(j)^0.5)^2*abs(u(k)-u(j));
 
        s2=Beta2(j)*nMat1(xx,j);
        ss2=ss2+s2;
end
 
ss1Mat(k)=ss1;
ss2Mat(k)=ss2;
 
dnt(xx,k)=alf*ss1-alf*ss2*nMat1(xx,k);
dnx(xx,k)=-u(k)/(xx*delx)*nMat1(xx,k)*delt;
 
%Updating the population matrix:
nMat(xx,k)=nMat1(xx,k)+delt*dnt(xx,k)+dnx(xx,k);
%The partilce which are removed from the system are added to the sediment bin 
%This used for latter calculation of the mass balance
nsink(xx,k)=-dnx(xx,k);
 
end
end
%The following loop prevent the number concentration matrix to become
%negative
for k=1:kk
if nMat(:,k)<1
    nMat(:,k)=0;
end
end
 
%calculation of the mass and mass concentration:
ms(:,1:kk)=RhoS*vsMat(:,1:kk);
c(:,1:kk)=nMat(:,1:kk).*ms(:,1:kk);
 
%?????????????????????????????????????????????????????????????????????????%
%Adjusting the delta_t based on the new numerical stability criteria 
%proposed by Babakhani et al., (2017):
%Finding the index of the bin to which the front of the concentration has moved
%in the current time step:
 
for k=kk:-1:1
    if nMat(:,k)>1
        break_k=k;
        break
    end
end
 
kumax=k+1;
if kumax>kk
    kumax=kk;
end
%
%Finding the maximum velocity that has occurred to this end:
umax=max(max(u(1:kumax)));
ss2Max=max(max(ss2Mat(1:kumax)));
deltstb=2/(ss2Max+abs(umax)/ds); 
 
delt=min([0.9*deltstb*deltmag delt1]);
%?????????????????????????????????????????????????????????????????????????%
 
%Limiting the lower value of Delta_t to the minimum Delta_t specified in the inputs:
if delt<mindelt
delt=mindelt;
end
 
%Extracting the model output times according to the observation times in order to 
%compare mdoel outputs with the observation data later:
if delt<delt1
    if t>=nobs*deltinitial
        ttemp=t;
        t=nobs*deltinitial;
        nobs=nobs+1; 
        delt=t-(ttemp-delt);
    end
end
 
%The following loop prevent the mass concentration matrix to become
%negative
for xx=ndelx;
    for k=1:kk
        if c(xx,k)<=0
            c(xx,k)=0;
        end
    end
end
 
%Calculating the geometric mean:
for xx=1:ndelx;
logtemp(1:kk)=log(2*aMat(xx,1:kk));
Dgeom(tt,xx)=exp(sum(c(xx,1:kk).*(2*aMat(xx,1:kk)).^3.*logtemp(1:kk))/...
    (sum(c(xx,1:kk).*(2*aMat(xx,1:kk)).^3)))*LaterSizeTransRatio;
size_ave(tt,xx)=Dgeom(tt,xx)/2; 
end
%The following lines are used to calculate times in shear force is used
%to simulate the condition of the vortex before the start of the
%measurements. They should be modified if this is going to be considered:
    tstartMat(pn,st)=t-tstarttempMat(1);
    st=st+1;
 %{  
    else
        if t>=T1
            if t<3*T
            T1=T1+T;
            end
        end
end
%}
    %{
if tstarttempMat(1)==t
        T1=t+T;
%lmb(1:kk)=lmb(1:kk);
        G=0
 
        for xx=1:ndelx
         c(xx,:)=c(xx,:)/sum(c(xx,:))*sum(c0bin(:));
        end
        nMat=c./ms;
end
%}
 
%Calculation of the mass balance:
if ndelx==1
delx=ds;
end
 
        for xx=1:ndelx
        Conc1(tt,xx)=sum(c(xx,:))./sum(c0bin(:));
        msinksum(tt,xx)=sum(nsink(xx,1:kk).*ms(xx,1:kk))*Across*delx;
        end
        massBal1(tt)=sum(sum(c(:,:)))*delx*Across;
        
%Increasing the speed by increasing the Delta_t after the initial stage of
% the aggregation is passed:
if t-tstarttempMat(1)>=T*RelaxationPortion
    if tamam0==0
delt1=delt1*deltfac/deltfacMatRelax;
mindelt=mindelt/deltfac*deltfacMatRelax;
    end
        tamam0=1;
end
      
%Extracting the data for size distribution at middle time:
if t-tstarttempMat(1)>=TT1
    if tamam==0
size_ditrib2(pn,1:kk/bin_resolut_fac)=aMat(mdpNdx,1:bin_resolut_fac:kk)*10^9*2;  %nm
No_size_ditrib2(caseNo,1:kk/bin_resolut_fac)=nMat(mdpNdx,1:bin_resolut_fac:kk)/sum(nMat(mdpNdx,1:bin_resolut_fac:kk));
Mass_size_dist2(caseNo,1:kk/bin_resolut_fac)=c(mdpNdx,1:bin_resolut_fac:kk)/sum(c(mdpNdx,1:bin_resolut_fac:kk));
    end
        tamam=1;
end
 
%Extracting the data for size distribution at the end time:
if t-tstarttempMat(1)>=TT2
        if tamam1==0
size_ditrib3(pn,1:kk/bin_resolut_fac)=aMat(mdpNdx,1:bin_resolut_fac:kk)*10^9*2; %nm
No_size_ditrib3(caseNo,1:kk/bin_resolut_fac)=nMat(mdpNdx,1:bin_resolut_fac:kk)/sum(nMat(mdpNdx,1:bin_resolut_fac:kk));
Mass_size_dist3(caseNo,1:kk/bin_resolut_fac)=c(mdpNdx,1:bin_resolut_fac:kk)/sum(c(mdpNdx,1:bin_resolut_fac:kk));
    end
        tamam1=1;
end
 
tMat(tt)=t;
 
%Limiting the maximum number of time steps to 500000:
if tt>500000
    Massage='Number of time steps exceeds 10000'
    break
end
 
%Showing the simulation progress, (percentage remained):
Run_Time_Remained_Percent=(T-t)/T*100;
 
 
%Terminating the run if any of the following occurs:
if isnan(t)==1
   Massage='Error: Instability problem, time is going wrong'
    break
end
 
if isreal(t)==0
   Massage='Error: Instability problem, time is going wrong (imaginary number)'
    break
end
 
end
%This was the end of the time step loop
%))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
%% 
%This for finding the real initial time when the initial vortex condition is involved:
earlyt(pn)=tstarttempMat(1);
ttstart=tt-st+2;
if st<=2
    st=tt;
    ttstart=1;
    tstartMat(pn,:)=tMat;
else
    st=st-1;
end
 
%Continue of the calculation of the mass balance error:
sinkCumulut=0;
for ttt=1:st
mass_sinkTotal(ttt)=sum(msinksum(ttstart-1+ttt,:));
sinkCumulut=sinkCumulut+mass_sinkTotal(ttt);
SedimentCumul(ttt)=sinkCumulut;
SedMassratio(ttt)=SedimentCumul(ttt)/massInitial*100;  %Percent
 
end
MassBalratio(1:st)=massBal1(ttstart:tt)/massInitial*100;  %Percent
MassBalratio(st)=MassBalratio(st);
SedMassratio(st)=SedMassratio(st);
finalMassBalance=MassBalratio(st)+SedMassratio(st);  % Should be 100 Percent
finalMassBalanceError(pn)=100-finalMassBalance  % Should be 100 Percent
 
%calculation of characteristics time (taw) (Chapter 6 of Elimelech et al.,
%1995):
taw=1/(nt0*4*1.381*10^(-23)*Temp/(3*Visc));
 
%Calculating the averaged size:
for ttt=1:st
size_out(ttt,1:ndelx)=size_ave(ttstart-1+ttt,1:ndelx);
end
 
%extracting the observation data from outputs
%tOutput=0:deltinitial:(ndelt1/deltfac-1)*deltinitial;
 
tOutput=Inputx;
 
for i=1:dataNo(pn)
tmp = abs(tstartMat(pn,1:st)-tOutput(i)*60);
[idx idx] = min(tmp); %index of closest value
t_indexoutput(i)=idx; %closest value
 
end
Size_Output=2*size_out(t_indexoutput,mdpNdx)*10^9;   %/gamma(2-(real(Df))^-1);   %nm
Size_Output=transpose(Size_Output); 
modeled=Size_Output;
 
for ttt=1:st
Conc(ttt,1:ndelx)=Conc1(ttstart-1+ttt,1:ndelx);
end
 
%Size distribution:
snapshotconc2=round(t_indexoutput(dataNo(pn))/2);
snapshotconc3=round(t_indexoutput(dataNo(pn)));
ConcInit(1:ndelx,caseNo)=Conc(1,1:ndelx);
ConcMid(1:ndelx,caseNo)=Conc(snapshotconc2,1:ndelx);
ConcEnd(1:ndelx,caseNo)=Conc(snapshotconc3,1:ndelx);
 
ConcOutput=0;
ConcOutput=Conc(t_indexoutput,mdpNdx);
ConcOutputMat(caseNo,1:dataNo(pn))=ConcOutput(1:dataNo(pn));
ConcMat(caseNo,1:dataNo(pn),1:ndelx)=ConcOutput(1:dataNo(pn));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plotting the results and calculation of the fitting R_Square:
 
    % R-Square Nash and Sutcliffe:
    meanoutput= mean(Inputy(1:dataNo(pn)));
    obsminumin= Inputy(1:dataNo(pn)) - meanoutput;
    obsminuminsq=obsminumin.*obsminumin;
    calcminuobs= transpose(modeled(1:dataNo(pn))) - Inputy(1:dataNo(pn));
    calcminuobssq= calcminuobs.*calcminuobs;
    %number of inputs
    elem=size(Inputy(1:dataNo(pn)));
    sumationo=0;
    sumationc=0;
    for hi=1:elem(1)
        sumationo=sumationo+obsminuminsq(hi);
        sumationc=sumationc+calcminuobssq(hi);
    end
    obsminuminsqs=sumationo;
    calcminuobssqs= sumationc;
    Rsq(pn)=1-(calcminuobssqs/obsminuminsqs)
 
 
maxsize(pn)=max(Inputy(1:dataNo(pn)));
RuningTimeMat(pn)=toc/60;   %min
end
 
%
%??????????????????????????????????????????????????????????????????????????%
%This is the end of the loop that model various experimental cases. The rest
%of the graphs are plotted separately as follows
%??????????????????????????????????????????????????????????????????????????%
 
 
%%
%$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
%Plotting the modelled concentration (dimensionless) vs observed count rate
%(dimensionless):
%fig2=figure;
 pn=1;
 
    if PlotNo>1
        caseNo=pn;
    end
 
    Inputx=Input2(1:dataNo(pn),3*caseNo-2);
    Inputy=Input2(1:dataNo(pn),3*caseNo-1)/Input2(1,3*caseNo-1);
    InputErr=Input2(:,3*caseNo)/Input2(1,3*caseNo-1);
    InputEr=InputErr;
    L=Inputy(:)<0;
    Inputx(L)=[];
    Inputy(L)=[];
    InputEr(L)=[];
    sizI=size(InputEr);
 
  nan=isnan(InputEr);
  nnan=nnz(nan);
  net=sizI(1)-nnan;
        panj=[2*pn:5:net];
        newInputEr=zeros(sizI(1));
        panjmat=InputEr(panj);
        newInputEr(sub2ind(size(InputEr), panj'))=panjmat;
 
    % R-Square Nash and Sutcliffe:
    meanoutput= mean(Inputy(1:dataNo(pn)));
    obsminumin= Inputy(1:dataNo(pn)) - meanoutput;
    obsminuminsq=obsminumin.*obsminumin;
    calcminuobs= transpose(ConcOutputMat(caseNo,(1:dataNo(pn)))) - Inputy(1:dataNo(pn));
    calcminuobssq= calcminuobs.*calcminuobs;
    %number of inputs
    elem=size(obsminuminsq);
    sumationo=0;
    sumationc=0;
    for hi=1:elem(1);
        sumationo=sumationo+obsminuminsq(hi);
        sumationc=sumationc+calcminuobssq(hi);
    end
    obsminuminsqs=sumationo;
    calcminuobssqs= sumationc;
 
    RsqConc(pn)=1-(calcminuobssqs/obsminuminsqs)
 
 
for pn=1:PlotNo
    if PlotNo>1
        caseNo=pn;
    end
FinalResults(1:9,pn)=[alfMat(1) DfMat(1) deltfacMat(1)...
    finalMassBalanceError(pn)  Rsq(pn)  RsqConc(pn)  RuningTimeMat(pn) LaterSizeTransRatioGeom(pn) deltmagMat(caseNo)];
 
end
 
TableItems={'alfa' 'Df' 'DeltaTFactor' 'MassBalanceErr' 'Rsq' 'Rsq_Conc.' 'RuningTime' 'SizeTransRatioGeom' 'Delt_t_magnification'};
 
dlmwrite(['Rep_Table_' num2str(OutputFileName)],transpose(FinalResults(:,1:PlotNo)),'delimiter',...
        '\t','precision',16,'roffset',1);
 
%printmat(FinalResults, FileName, 'DeltaTFactor IS Zeta Df MassBalanceErr Rsq RuningTime SizeTransRatioGeom', '1 2 3 4 5');
 
%save('Opt_Results.mat', 'Outcomes');
 
Rsqout=Rsq;
RsqConcout=RsqConc;
finalMassBalanceErrorout=finalMassBalanceError;
 
 end

